package org.processmining.acceptingpetrinetreductionlayouter.models;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Collection;

import org.processmining.acceptingpetrinetreductionlayouter.workspaces.ReductionWorkspace;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class NodeBlock implements Block {

	private PetrinetNode node;
	private Collection<PetrinetNode> prePostNodes;
	private ReductionWorkspace ws;
	private double hSpace;

	public NodeBlock(PetrinetNode node, ReductionWorkspace ws) {
		this.node = node;
		this.ws = ws;
		hSpace = ws.parameters.getHorizontalSpace() * ws.SCALE;
		prePostNodes = new ArrayList<PetrinetNode>(ws.presets.get(node));
		prePostNodes.addAll(ws.postsets.get(node));
	}

	public int width(double hSpace) {
		this.hSpace = hSpace;
		Dimension dim = (ws.reversePlaceMap.keySet().contains(node)
				? ws.layoutConnection.getSize(ws.reversePlaceMap.get(node)) : new Dimension(1, 1));
		return (int) (ws.SCALE * dim.getWidth());
	}
	
	public int[] height() {
		Dimension dim = (ws.reversePlaceMap.keySet().contains(node)
				? ws.layoutConnection.getSize(ws.reversePlaceMap.get(node)) : new Dimension(1, 1));
		int[] height = new int[2];
		height[0] = (int) (ws.SCALE * dim.getHeight()) / 2;
		height[1] = (int) (ws.SCALE * dim.getHeight()) - height[0];
		return height;
	}

	public int getNofLayers() {
		return 1;
	}
	
	public void doLayout(ReductionWorkspace ws, Pair<Double, Double> position, boolean isReversed, double hSpace) {
		ws.layout.put(node, position);
	}

	public boolean doSort(ReductionWorkspace ws) {
		return false;
	}
	
	public Collection<PetrinetNode> getNodes(boolean addPrePostNodes) {
		Collection<PetrinetNode> nodes = new ArrayList<PetrinetNode>();
		nodes.add(node);
		if (addPrePostNodes) {
			nodes.addAll(prePostNodes);
		}
		return nodes;
	}
	
	public boolean doSortLoops(boolean isReversed) {
		return false;
	}
	
	public Transition getFirstVisibleTransition(ReductionWorkspace ws) {
		if (ws.reversePlaceMap.get(node) instanceof Transition) {
			Transition transition = (Transition) ws.reversePlaceMap.get(node);
			if (!transition.isInvisible()) {
				return transition;
			}
		}
		return null;
	}

	public Transition getLastVisibleTransition(ReductionWorkspace ws) {
		return getFirstVisibleTransition(ws);
	}
	
	public void getTransitionPairs(ReductionWorkspace ws, Collection<Pair<Transition, Transition>> pairs) {		
	}
	
	public void doPrint(int indent) {
		String s = "";
		for (int i = 0; i < indent; i++) {
			s += "\t";
		}
		int[] height = height();
		s += "Block [" + width(hSpace) + "," + height[0] + "+" + height[1] + "] " + node.getLabel();
		System.out.println(s);
	}
}
