package org.processmining.acceptingpetrinetreductionlayouter.models;

import java.util.Collection;
import java.util.HashSet;

import org.processmining.acceptingpetrinetreductionlayouter.workspaces.ReductionWorkspace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class PlaceReduction implements Reduction {

	private Place place;
	private Place otherPlace;
	private double score = -1.0;

	public Place getPlace() {
		return place;
	}

	public void setPlace(Place place) {
		this.place = place;
	}

	public Place getOtherPlace() {
		return otherPlace;
	}

	public void setOtherPlace(Place otherPlace) {
		this.otherPlace = otherPlace;
	}

	public void apply(ReductionWorkspace ws) {
		// Reroute connections to and from place to and from otherPlace.
		for (PetrinetNode node : ws.presets.get(place)) {
			Transition transition = (Transition) node;
			ws.workNet.getNet().addArc(transition, otherPlace);
		}
		for (PetrinetNode node : ws.postsets.get(place)) {
			Transition transition = (Transition) node;
			ws.workNet.getNet().addArc(otherPlace, transition);
		}
		// Connections have bee rerouted, now remove the place.
		ws.workNet.getNet().removePlace(place);
		// Construct a vertical block.
		Block block = new VerticalBlock(ws.layoutMap.get(place),
				ws.layoutMap.get(otherPlace), ws, false);
		ws.layoutMap.put(otherPlace, block);
		ws.update();
	}

	public double score(ReductionWorkspace ws) {
		if (score < 0.0) {
			Collection<PetrinetNode> prePrePlaces = new HashSet<PetrinetNode>();
			Collection<PetrinetNode> anotherPrePrePlaces = new HashSet<PetrinetNode>();
			Collection<PetrinetNode> postPostPlaces = new HashSet<PetrinetNode>();
			Collection<PetrinetNode> anotherPostPostPlaces = new HashSet<PetrinetNode>();
			for (PetrinetNode node : ws.presets.get(place)) {
				prePrePlaces.addAll(ws.presets.get(node));
			}
			for (PetrinetNode node : ws.presets.get(otherPlace)) {
				anotherPrePrePlaces.addAll(ws.presets.get(node));
			}
			for (PetrinetNode node : ws.postsets.get(place)) {
				postPostPlaces.addAll(ws.postsets.get(node));
			}
			for (PetrinetNode node : ws.postsets.get(otherPlace)) {
				anotherPostPostPlaces.addAll(ws.postsets.get(node));
			}
			Collection<PetrinetNode> preUnion = new HashSet<PetrinetNode>(prePrePlaces);
			Collection<PetrinetNode> preDiff = new HashSet<PetrinetNode>(prePrePlaces);
			Collection<PetrinetNode> postUnion = new HashSet<PetrinetNode>(postPostPlaces);
			Collection<PetrinetNode> postDiff = new HashSet<PetrinetNode>(postPostPlaces);
			preUnion.addAll(anotherPrePrePlaces);
			preDiff.retainAll(anotherPrePrePlaces);
			postUnion.addAll(anotherPostPostPlaces);
			postDiff.retainAll(anotherPostPostPlaces);
			score = 0.98 * (1.0 * preDiff.size() + postDiff.size()) / (preUnion.size() + postUnion.size());
		}
		return score;
	}

	public boolean hasOptimalScore(ReductionWorkspace ws) {
		return score(ws) == 0.98;
	}

	public String toString() {
		return "Place(" + place + "," + otherPlace + ")";
	}

}
