package org.processmining.acceptingpetrinetreductionlayouter.models;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.processmining.acceptingpetrinetreductionlayouter.utils.NodeComparator;
import org.processmining.acceptingpetrinetreductionlayouter.workspaces.ReductionWorkspace;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class TransitionReduction implements Reduction {

	/*
	 * The reduced transition.
	 */
	private Transition transition;
	/*
	 * The transitions on which the reduced transition was nonnegatively
	 * linearly dependent.
	 */
	private List<Transition> transitions;
	/*
	 * Whether reversed.
	 */
	private boolean isReversed;

	private NodeComparator nodeComparator = new NodeComparator();

	public Transition getTransition() {
		return transition;
	}

	public void setTransition(Transition transition) {
		this.transition = transition;
	}

	public List<Transition> getTransitions() {
		return transitions;
	}

	public void setTransitions(Collection<Transition> transitions) {
		this.transitions = new ArrayList<Transition>(transitions);
		Collections.sort(this.transitions, nodeComparator);
	}

	public boolean isReversed() {
		return isReversed;
	}

	public void setReversed(boolean isReversed) {
		this.isReversed = isReversed;
	}

	public void apply(ReductionWorkspace ws) {
//		if (transitions.size() < 2) {
			ws.workNet.getNet().removeTransition(transition);
//		}
		// Update layout
		if (transitions.size() == 0) {
			Place place = (Place) (ws.presets.get(transition).isEmpty() ? ws.postsets.get(transition).iterator().next()
					: ws.presets.get(transition).iterator().next());
			Block block = new VerticalBlock(ws.layoutMap.get(place),
					ws.layoutMap.get(transition), ws, false, VerticalBlock.VALIGNTOP);
//			block = new DeselEsparzaVerticalBlock(ws.layoutMap.get(transition), block, ws, false);
//			ws.reverseTransitionMap.get(transition).getAttributeMap().put(AttributeMap.EDGECOLOR, Color.BLUE);

			ws.layoutMap.put(place, block);
		} else if (transitions.size() > 1){
			List<PetrinetNode> prePlaces = new ArrayList<PetrinetNode>();
			List<PetrinetNode> postPlaces = new ArrayList<PetrinetNode>();
			for (Transition t : transitions) {
				prePlaces.addAll(ws.presets.get(t));
				postPlaces.addAll(ws.postsets.get(t));
			}
			List<PetrinetNode> nodes = new ArrayList<PetrinetNode>(prePlaces);
			nodes.retainAll(postPlaces);
			
			ws.hardTransitions.add(transition);
			
//			Collection<PetrinetNode> preTransitions = new HashSet<PetrinetNode>();
//			Collection<PetrinetNode> postTransitions = new HashSet<PetrinetNode>();
//			for (PetrinetNode node : nodes) {
//				preTransitions.addAll(ws.presets.get(node));
//				postTransitions.addAll(ws.postsets.get(node));
//			}
//			preTransitions.removeAll(transitions);
//			postTransitions.removeAll(transitions);
//			for (PetrinetNode node : nodes) {
//				for (PetrinetNode preTransition : ws.presets.get(node)) {
//					if (preTransitions.contains(preTransition)) {
//						ws.workNet.getNet().removeArc(preTransition, node);
//						ws.workNet.getNet().addArc((Transition) preTransition, (Place) ws.presets.get(transition).get(0));
//					}
//				}
//				for (PetrinetNode postTransition : ws.postsets.get(node)) {
//					if (postTransitions.contains(postTransition)) {
//						ws.workNet.getNet().removeArc(node, postTransition);
//						ws.workNet.getNet().addArc((Place) ws.postsets.get(transition).get(0), (Transition) postTransition);
//					}
//				}
//			}
			
			nodes.addAll(transitions);
			PetrinetNode bestNode = nodes.iterator().next();
			int bestHeight[] = ws.layoutMap.get(bestNode).height();
			int bestWidth = ws.layoutMap.get(bestNode).width(ws.parameters.getHorizontalSpace() * ws.SCALE);
			for (PetrinetNode node : nodes) {
				int height[] = ws.layoutMap.get(node).height();
				int width = ws.layoutMap.get(node).width(ws.parameters.getHorizontalSpace() * ws.SCALE);
				if (isBetter(width, height, bestWidth, bestHeight)) {
					bestNode = node;
					bestHeight = height;
					bestWidth = width;
				}
			}
			
//			ws.reverseTransitionMap.get(transition).getAttributeMap().put(AttributeMap.EDGECOLOR, Color.BLUE);
			
			Block block = new VerticalBlock(ws.layoutMap.get(transition), ws.layoutMap.get(bestNode), ws, isReversed, VerticalBlock.VALIGNBOTTOM);
			ws.layoutMap.put(bestNode, block);
		} else if (!isReversed) {
			if (!ws.presets.get(transition).isEmpty()) {
				Place prePlace = (Place) ws.presets.get(transition).iterator().next();
				Collection<PetrinetNode> siblingTransitions = ws.postsets.get(prePlace);
				siblingTransitions.retainAll(transitions);
				Transition siblingTransition = (Transition) siblingTransitions.iterator().next();
				Block block = new VerticalBlock(ws.layoutMap.get(siblingTransition),
						ws.layoutMap.get(transition), ws, false);
				ws.layoutMap.put(siblingTransition, block);
			} else if (!ws.postsets.get(transition).isEmpty()) {
				Place postPlace = (Place) ws.postsets.get(transition).iterator().next();
				Collection<PetrinetNode> siblingTransitions = ws.presets.get(postPlace);
				siblingTransitions.retainAll(transitions);
				Transition siblingTransition = (Transition) siblingTransitions.iterator().next();
				Block block = new VerticalBlock(ws.layoutMap.get(siblingTransition),
						ws.layoutMap.get(transition), ws, false);
				ws.layoutMap.put(siblingTransition, block);
			}
//			ws.reverseTransitionMap.get(transition).getAttributeMap().put(AttributeMap.EDGECOLOR, Color.BLUE);
		} else {
			if (!ws.postsets.get(transition).isEmpty()) {
				Place prePlace = (Place) ws.postsets.get(transition).iterator().next();
				Collection<PetrinetNode> siblingTransitions = ws.postsets.get(prePlace);
				siblingTransitions.retainAll(transitions);
				Transition siblingTransition = (Transition) siblingTransitions.iterator().next();
				Block block = new VerticalBlock(ws.layoutMap.get(siblingTransition),
						ws.layoutMap.get(transition), ws, true);
				ws.layoutMap.put(siblingTransition, block);
			} else if (!ws.presets.get(transition).isEmpty()) {
				Place postPlace = (Place) ws.presets.get(transition).iterator().next();
				Collection<PetrinetNode> siblingTransitions = ws.presets.get(postPlace);
				siblingTransitions.retainAll(transitions);
				Transition siblingTransition = (Transition) siblingTransitions.iterator().next();
				Block block = new VerticalBlock(ws.layoutMap.get(siblingTransition),
						ws.layoutMap.get(transition), ws, true);
				ws.layoutMap.put(siblingTransition, block);
			}
//			ws.reverseTransitionMap.get(transition).getAttributeMap().put(AttributeMap.EDGECOLOR, Color.BLUE);
		}
		ws.update();
	}

	private boolean isBetter(int width, int height[], int bestWidth, int bestHeight[]) {
//		return (width * (height[0] + height[1]) > bestWidth * (bestHeight[0] + bestHeight[1]));
		return (height[1] > bestHeight[1] || (height[1] == bestHeight[1] && width > bestWidth));
	}
	
	public String toString() {
		return "Transition[" + isReversed + "](" + transition + "," + transitions + ")";
	}

	public double score(ReductionWorkspace ws) {
		if (transitions.size() == 0) {
			return 1.0;
		}
		for (PetrinetNode node : ws.presets.get(transition)) {
			if (ws.postsets.get(node).size() == 1) {
				return 0.0;
			}
		}
		for (PetrinetNode node : ws.postsets.get(transition)) {
			if (ws.presets.get(node).size() == 1) {
				return 0.0;
			}
		}
		return 1.0 / (transitions.size());
	}

	public boolean hasOptimalScore(ReductionWorkspace ws) {
		return score(ws) == 1.0;
	}
}
