package org.processmining.acceptingpetrinetreductionlayouter.models;

import java.util.ArrayList;
import java.util.Collection;

import org.processmining.acceptingpetrinetreductionlayouter.workspaces.ReductionWorkspace;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;

public class VerticalBlock implements Block {

	private final int NOFLAYOUTS = 2;
	private Block[] layouts;
	private ReductionWorkspace ws;
	private boolean[] isReversed;
	public final static int VALIGNCENTER = 0;
	public final static int VALIGNTOP = 1;
	public final static int VALIGNBOTTOM = 2;
	private int vAlign;
	private double hSpace;

	public VerticalBlock(Block topLayout, Block bottomLayout, ReductionWorkspace ws, boolean isReversed) {
		this(topLayout, bottomLayout, ws, isReversed, VALIGNCENTER);
	}

	public VerticalBlock(Block topLayout, Block bottomLayout, ReductionWorkspace ws, boolean isReversed, int vAlign) {
		layouts = new Block[NOFLAYOUTS];
		layouts[0] = topLayout;
		layouts[1] = bottomLayout;
		this.ws = ws;
		hSpace = ws.parameters.getHorizontalSpace() * ws.SCALE;
		this.isReversed = new boolean[NOFLAYOUTS];
		this.isReversed[0] = false;
		this.isReversed[1] = isReversed;
		this.vAlign = vAlign;
	}

	public int width(double hSpace) {
		this.hSpace = hSpace;
		int width = 0;
		for (int i = 0; i < NOFLAYOUTS; i++) {
			width = Math.max(width, layouts[i].width(hSpace));
		}
		return width;
	}

	public int[] height() {
		int[] height = new int[2];
		int[] heightTop = layouts[0].height();
		int[] heightBottom = layouts[1].height();
		height[0] = 0;
		height[1] = 0;
		switch (vAlign) {
			case VALIGNCENTER :
				height[0] = (int) (heightTop[0] + heightTop[1] + (ws.parameters.getVerticalSpace() * ws.SCALE) + heightBottom[0] + heightBottom[1]) / 2;
				break;
			case VALIGNTOP :
				height[0] = heightTop[0];
				break;
			case VALIGNBOTTOM :
				height[0] = (int) (heightTop[0] + heightTop[1] + (ws.parameters.getVerticalSpace() * ws.SCALE) + heightBottom[0]);
				break;
		}
		height[1] = (int) (heightTop[0] + heightTop[1] + (ws.parameters.getVerticalSpace() * ws.SCALE) + heightBottom[0] + heightBottom[1]) - height[0];
		return height;
	}

	public int getNofLayers() {
		int n = 0;
		for (int i = 0; i < NOFLAYOUTS; i++) {
			n = Math.max(n, layouts[i].getNofLayers());
		}
		return n;
	}

	public void doLayout(ReductionWorkspace ws, Pair<Double, Double> position, boolean isReversed, double hSpace) {
		int width = width(hSpace);
		for (int i = 0; i < NOFLAYOUTS; i++) {
			int nofLayers = layouts[i].getNofLayers();
			if (nofLayers == 1) {
				double offset = (width - layouts[i].width(hSpace)) / 2;
				Pair<Double, Double> offsetPosition = new Pair<Double, Double>(position.getFirst() + offset,
						position.getSecond());
				layouts[i].doLayout(ws, offsetPosition, (this.isReversed[i] ? !isReversed : isReversed), hSpace);
				int height[] = layouts[i].height();
				position = new Pair<Double, Double>(position.getFirst(),
						position.getSecond() + height[0] + height[1] + (ws.parameters.getVerticalSpace() * ws.SCALE));
			} else {
				double offset = (width - layouts[i].width(hSpace)) / ((nofLayers - 1) * 2);
				Pair<Double, Double> offsetPosition = new Pair<Double, Double>(position.getFirst(),
						position.getSecond());
				layouts[i].doLayout(ws, offsetPosition, (this.isReversed[i] ? !isReversed : isReversed),
						hSpace + 2 * offset);
				int height[] = layouts[i].height();
				position = new Pair<Double, Double>(position.getFirst(),
						position.getSecond() + height[0] + height[1] + (ws.parameters.getVerticalSpace() * ws.SCALE));
			}
		}
	}

	public boolean doSort(ReductionWorkspace ws) {
		boolean swapped = false;

		Collection<PetrinetNode> nodes0 = layouts[0].getNodes(true);
		nodes0.removeAll(layouts[0].getNodes(false));
		double sumY0 = 0.0;
		for (PetrinetNode node0 : nodes0) {
			sumY0 += ws.layout.get(node0).getSecond();
		}
		if (!nodes0.isEmpty()) {
			sumY0 /= nodes0.size();
		}
		Collection<PetrinetNode> nodes1 = layouts[1].getNodes(true);
		nodes1.removeAll(layouts[1].getNodes(false));
		double sumY1 = 0.0;
		for (PetrinetNode node1 : nodes1) {
			sumY1 += ws.layout.get(node1).getSecond();
		}
		if (!nodes1.isEmpty()) {
			sumY1 /= nodes1.size();
		}
		if (sumY0 > sumY1 + ws.threshold) {
			System.out.println("[VerticalBlock] Swapping 0/1 " + layouts[0].getNodes(false) + " and "
					+ layouts[1].getNodes(false));
			Block block = layouts[1];
			layouts[1] = layouts[0];
			layouts[0] = block;
			boolean b = isReversed[1];
			isReversed[1] = isReversed[0];
			isReversed[0] = b;
			swapped = true;
		}

		if (layouts[0] instanceof VerticalBlock) {
			VerticalBlock layouts0 = (VerticalBlock) layouts[0];
			Collection<PetrinetNode> nodes01 = layouts0.layouts[1].getNodes(true);
			nodes01.removeAll(layouts0.layouts[1].getNodes(false));
			double sumY01 = 0.0;
			for (PetrinetNode node01 : nodes01) {
				sumY01 += ws.layout.get(node01).getSecond();
			}
			if (!nodes01.isEmpty()) {
				sumY01 /= nodes01.size();
			}
			if (sumY01 > sumY1 + ws.threshold) {
				System.out.println("[VerticalBlock] Swapping 0.1/1 " + layouts0.layouts[1].getNodes(false) + " and "
						+ layouts[1].getNodes(false));
				Block block = layouts0.layouts[1];
				layouts0.layouts[1] = layouts[1];
				layouts[1] = block;
				boolean b = layouts0.isReversed[1];
				layouts0.isReversed[1] = isReversed[1];
				isReversed[1] = b;
				swapped = true;
			}
		}

		if (layouts[1] instanceof VerticalBlock) {
			VerticalBlock layouts1 = (VerticalBlock) layouts[1];
			Collection<PetrinetNode> nodes10 = layouts1.layouts[0].getNodes(true);
			nodes10.removeAll(layouts1.layouts[0].getNodes(false));
			double sumY10 = 0.0;
			for (PetrinetNode node10 : nodes10) {
				sumY10 += ws.layout.get(node10).getSecond();
			}
			if (!nodes10.isEmpty()) {
				sumY10 /= nodes10.size();
			}
			if (sumY0 > sumY10 + ws.threshold) {
				System.out.println("[VerticalBlock] Swapping 0/1.0 " + layouts[0].getNodes(false) + " and "
						+ layouts1.layouts[0].getNodes(false));
				Block block = layouts1.layouts[0];
				layouts1.layouts[0] = layouts[0];
				layouts[0] = block;
				boolean b = layouts1.isReversed[0];
				layouts1.isReversed[0] = isReversed[0];
				isReversed[0] = b;
				swapped = true;
			}
		}

		return layouts[0].doSort(ws) || layouts[1].doSort(ws) || swapped;
	}

	public Collection<PetrinetNode> getNodes(boolean addPrePostNodes) {
		Collection<PetrinetNode> nodes = new ArrayList<PetrinetNode>();
		for (int i = 0; i < NOFLAYOUTS; i++) {
			nodes.addAll(layouts[i].getNodes(addPrePostNodes));
		}
		return nodes;
	}

	public boolean doSortLoops(boolean isRev) {
		if (combine(isRev, isReversed[0]) && combine(isRev, !isReversed[1])) {
			Block layout = layouts[1];
			layouts[1] = layouts[0];
			layouts[0] = layout;
			isReversed[1] = combine(isRev, true);
			isReversed[0] = combine(isRev, false);
			return true;
		} else if (combine(isRev, !isReversed[1]) && layouts[0] instanceof VerticalBlock
				&& combine(isRev, ((VerticalBlock) layouts[0]).isReversed[1])) {
			Block layout = layouts[1];
			layouts[1] = ((VerticalBlock) layouts[0]).layouts[1];
			((VerticalBlock) layouts[0]).layouts[1] = layout;
			isReversed[1] = combine(isRev, true);
			((VerticalBlock) layouts[0]).isReversed[1] = combine(isRev, false);
			return true;
		} else if (combine(isRev, isReversed[0]) && layouts[1] instanceof VerticalBlock
				&& !combine(isRev, ((VerticalBlock) layouts[1]).isReversed[0])) {
			Block layout = layouts[0];
			layouts[0] = ((VerticalBlock) layouts[1]).layouts[0];
			((VerticalBlock) layouts[1]).layouts[0] = layout;
			isReversed[0] = combine(isRev, false);
			((VerticalBlock) layouts[1]).isReversed[0] = combine(isRev, true);
			return true;
		}
		return layouts[0].doSortLoops(combine(isRev, isReversed[0]))
				|| layouts[1].doSortLoops(combine(isRev, isReversed[1]));
	}

	private boolean combine(boolean isRev, boolean isReversed) {
		return isRev ? !isReversed : isReversed;
	}

	public Transition getFirstVisibleTransition(ReductionWorkspace ws) {
		return null;
	}

	public Transition getLastVisibleTransition(ReductionWorkspace ws) {
		return null;
	}

	public void getTransitionPairs(ReductionWorkspace ws, Collection<Pair<Transition, Transition>> pairs) {
		for (int i = 0; i < NOFLAYOUTS; i++) {
			if (layouts[i].getNofLayers() > 30) {
				System.out.println("[VerticalBlock] NofLayouts = " + layouts[i].getNofLayers());
				Transition firstTransition = layouts[i].getFirstVisibleTransition(ws);
				Transition lastTransition = layouts[i].getLastVisibleTransition(ws);
				if (firstTransition != null && lastTransition != null && firstTransition != lastTransition) {
					pairs.add(new Pair<Transition, Transition>(firstTransition, lastTransition));
				}
				//			if (layouts[i] instanceof VerticalBlock) {
				layouts[i].getTransitionPairs(ws, pairs);
				//			}
			}
		}
	}

	public void doPrint(int indent) {
		String s = "";
		for (int i = 0; i < indent; i++) {
			s += "\t";
		}
		int[] height = height();
		s += "Vertical [" + width(hSpace) + "," + height[0] + "+" + height[1] + "] " + isReversed[0] + " "
				+ isReversed[1];
		System.out.println(s);
		layouts[0].doPrint(indent + 1);
		layouts[1].doPrint(indent + 1);
	}
}
