package org.processmining.acceptingpetrinetreductionlayouter.plugins;

import javax.swing.JComponent;

import org.processmining.acceptingpetrinetreductionlayouter.parameters.LayoutParameters;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.semantics.petrinet.Marking;

public abstract class AbstractVisualizePlugin {

	public abstract LayoutParameters getParameters();
	
	public JComponent visualize(PluginContext context, Petrinet net) {

		Petrinet blockNet = (new LayoutPlugin()).run(context, net, getParameters());
		Marking m = new Marking();
		ConnectionManager cm = context.getConnectionManager();
		// Try to see if there is an initial marking available for visualization
		try {
			for (InitialMarkingConnection mnc : cm.getConnections(InitialMarkingConnection.class, context, blockNet)) {
				m = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);

			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
		}

		//		return PetriNetVisualization.getComponent(context, n, m);

		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		for (Place p : m) {
			String label = "";
			char bullet = '\u2022';
			int n = m.occurrences(p);
			if ((Boolean) p.getAttributeMap().get(AttributeMap.SHOWLABEL)) {
				if (n == 1) {
					label = "" + bullet + p.getLabel();
				} else {
					label = "" + n + bullet + p.getLabel();
				}
			} else {
				if (n == 1) {
					label = "<html><h2>" + bullet + "</h2><html>";
				} else if (n == 2) {
					label = "<html><h2>" + bullet + bullet + "</h2><html>";
				} else if (n == 3) {
					label = "<html><h2>" + bullet + bullet + bullet + "</h2><html>";
				} else {
					label = "" + n + bullet;
				}
			}
			map.putViewSpecific(p, AttributeMap.LABEL, label);
			map.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			if (n > 0) {
				map.putViewSpecific(p, AttributeMap.SHOWLABEL, true);
			}
		}

		return ProMJGraphVisualizer.instance().visualizeGraph(context, blockNet, map);
	}
}
