package org.processmining.acceptingpetrinetreductionlayouter.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.acceptingpetrinet.plugins.PackPetrinetPlugin;
import org.processmining.acceptingpetrinet.plugins.UnpackPetrinetPlugin;
import org.processmining.acceptingpetrinetreductionlayouter.algorithms.LayoutAlgorithm;
import org.processmining.acceptingpetrinetreductionlayouter.dialogs.LayoutDialog;
import org.processmining.acceptingpetrinetreductionlayouter.parameters.LayoutParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Block Layout Petri net", parameterLabels = { "Petri net", "Parameters" }, returnLabels = {
		"Net w/ Block Layout" }, returnTypes = { Petrinet.class }, userAccessible = true)
public class LayoutPlugin extends LayoutAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Default", requiredParameterLabels = { 0 })
	public Petrinet runDefault(UIPluginContext context, Petrinet net) {

		LayoutParameters parameters = new LayoutParameters();
		LayoutDialog dialog = new LayoutDialog(parameters);
		int n = 0;
		String[] title = { "Configure Block Layout" };
		InteractionResult result = InteractionResult.NEXT;
		while (result != InteractionResult.FINISHED) {
			result = context.showWizard(title[n], n == 0, n == 0, dialog.getPanel(n));
			if (result == InteractionResult.NEXT) {
				n++;
			} else if (result == InteractionResult.PREV) {
				n--;
			} else if (result != InteractionResult.FINISHED) {
				return null;
			}
		}
		
		return run(context, net, parameters);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Default", requiredParameterLabels = { 0 })
	public Petrinet runDefault(PluginContext context, Petrinet net) {

		return run(context, net, new LayoutParameters());
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Default", requiredParameterLabels = { 0, 1 })
	public Petrinet run(PluginContext context, Petrinet net, LayoutParameters parameters) {

		return (new UnpackPetrinetPlugin()).runDefault(context, apply(context, (new PackPetrinetPlugin().runDefault(context, net)), parameters));
	}

}